@@ -6,6 +6,7 @@ import re
 import sys
 import argparse
 import typing as t
+import warnings
 from argparse import Namespace
 from contextlib import contextmanager
 from io import StringIO
@@ -13,6 +14,7 @@ from pathlib import Path
 from traceback import format_exception
 from warnings import warn
 import signal
+import logging
 
 import docstring_parser as dp
 import pandas as pd
@@ -24,6 +26,7 @@ import cv2 as cv
 
 from .params import PrjParam
 from .typeoverloads import FilePath
+from .constants import PrjEnums
 
 # -----
 # For Pyyaml
@@ -153,10 +156,83 @@ def create_addMenuAct(mainWin: QtWidgets.QWidget, parentMenu: QtWidgets.QMenu, t
     return act
 
 
+old_showwarning = warnings.showwarning
 old_sys_except_hook = sys.excepthook
 usingPostponedErrors = False
 _eType = t.Union[t.Type[Exception], t.Type[Warning]]
 
+class AppLogger(logging.Logger):
+  nonCriticalErrors = ()
+
+  old_showwarning = None
+  old_sys_excepthook = None
+
+  def logLater(self, msg, *args, **kwargs):
+    # Define local function to avoid uncollected garbage on pyside2
+    def doLog():
+      self.log(msg, *args, **kwargs)
+    QtCore.QTimer.singleShot(0, doLog)
+
+  def attention(self, msg, *args, **kwargs):
+    return self.log(PrjEnums.LOG_LVL_ATTN, msg, *args, **kwargs)
+
+  def registerExceptions(self, win: QtWidgets.QMainWindow=None,
+                                 nonCriticalErrors: t.Tuple[_eType] = ()):
+    from .widgets import DialogHandler
+    self.old_sys_excepthook = sys.excepthook
+    self.addHandler(DialogHandler(logging.WARNING, win))
+    self.nonCriticalErrors = nonCriticalErrors
+
+    signal.signal(signal.SIGINT, signal.SIG_DFL)
+    sys.excepthook = self.exceptWithLog
+
+  def deregisterExceptions(self):
+    sys.excepthook = self.old_sys_excepthook
+
+  def registerWarnings(self):
+    self.old_showwarning = warnings.showwarning
+    warnings.showwarning = self.warnWithLog
+
+  def warnWithLog(self, message, category, filename, lineno, file=None, line=None):
+    """Copied logic from logging.captureWarnings implementation, but with short and long messages enabled"""
+    if file is not None:
+      if self.old_showwarning is not None:
+        self.old_showwarning(message, category, filename, lineno, file, line)
+    else:
+      detailedMsg = warnings.formatwarning(message, category, filename, lineno, line)
+      self.warning(message, extra={'detailed': detailedMsg})
+
+
+  def exceptWithLog(self, etype, evalue, tb):
+    # Allow sigabort to kill the app
+    app = QtWidgets.QApplication.instance()
+    if etype in [KeyboardInterrupt, SystemExit]:
+      app.exit(1)
+      app.processEvents()
+      raise
+
+    if issubclass(etype, self.nonCriticalErrors):
+      level = PrjEnums.LOG_LVL_ATTN
+    else:
+      level = logging.CRITICAL
+    self.log(level, '', exc_info=(etype, evalue, tb))
+
+  @classmethod
+  def getAppLogger(cls, name=''):
+    """
+    logging.getLogger only can spawn Logger classes, so this method includes a temporary override of the spawned
+    class so that an AppLogger can be registered instead.
+    """
+    oldCls = logging.getLoggerClass()
+    try:
+      logging.setLoggerClass(cls)
+      # False positive, logger class is overridden
+      # noinspection PyTypeChecker
+      logger: cls = logging.getLogger(name)
+    finally:
+      logging.setLoggerClass(oldCls)
+    return logger
+
 def makeExceptionsShowDialogs(win: QtWidgets.QMainWindow,
                               nonCritErrs: t.Tuple[_eType]=(Warning,)):
   """
@@ -166,8 +242,10 @@ def makeExceptionsShowDialogs(win: QtWidgets.QMainWindow,
   Use with caution! Maybe the application *should* crash on an error, but this will
   prevent that from happening.
   """
+  warn('This functionality is deprecated, please use an AppLogger with registerExceptionsToLogger instead',
+       DeprecationWarning, stacklevel=2)
   app = QtWidgets.QApplication.instance()
-  from utilitys.widgets import ScrollableErrorDialog
+  from utilitys.widgets import ScrollableMessageDialog
 
   # Procedure taken from https://stackoverflow.com/a/40674244/9463643
   def new_except_hook(etype, evalue, tb):
@@ -178,8 +256,8 @@ def makeExceptionsShowDialogs(win: QtWidgets.QMainWindow,
       raise
     msgWithTrace = html.escape(''.join(format_exception(etype, evalue, tb)))
     msgWithoutTrace = html.escape(str(evalue))
-    dlg = ScrollableErrorDialog(win, notCritical=issubclass(etype, nonCritErrs),
-                                msgWithTrace=msgWithTrace, msgWithoutTrace=msgWithoutTrace)
+    dlg = ScrollableMessageDialog(win, notCritical=issubclass(etype, nonCritErrs),
+                                  msgWithTrace=msgWithTrace, msgWithoutTrace=msgWithoutTrace)
     dlg.show()
     dlg.exec_()
   def patch_excepthook():
@@ -200,6 +278,9 @@ def restoreExceptionBehavior():
   app.processEvents()
 
 def raiseErrorLater(err: Exception):
+  warn('This functionality is deprecated, please use an AppLogger.logLater("CRITICAL", ...) instead'
+       ' (or any other log level)',
+       DeprecationWarning, stacklevel=2)
   # Fire immediately if not in gui mode
   if not usingPostponedErrors:
     raise err
@@ -209,6 +290,9 @@ def raiseErrorLater(err: Exception):
   QtCore.QTimer.singleShot(0, _raise)
 
 def warnLater(msg: str, type_=Warning):
+  warn('This functionality is deprecated, please use an AppLogger.logLater("ATTN", ...) instead'
+       ' (or any other log level)',
+       DeprecationWarning, stacklevel=2)
   if not usingPostponedErrors:
     warn(msg, type_, stacklevel=2)
   else:
@@ -308,7 +392,7 @@ def paramValues(param: Parameter, includeDefaults=False, includeKeys=()):
       outDict[param.name()] = inner
   return outDict
 
-def paramDictWithOpts(param: Parameter, addList: t.List[str]=None, addTo: t.List[type(Parameter)]=None,
+def paramDictWithOpts(param: Parameter, addList: t.List[str]=None, addTo: t.List[t.Type[Parameter]]=None,
                       removeList: t.List[str]=None):
   """
   Allows customized alterations to which portions of a pyqtgraph parameter will be saved
@@ -367,7 +451,7 @@ def params_flattened(param: Parameter):
 def flexibleParamTree(treeParams: t.Union[t.List[Parameter], Parameter]=None, showTop=True, setTooltips=True,
                       resizeNameCol=True):
   tree = ParameterTree()
-  tree.setTextElideMode(QtCore.Qt.ElideRight)
+  tree.setTextElideMode(QtCore.Qt.TextElideMode.ElideRight)
   tree.header().setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
   if isinstance(treeParams, Parameter):
     treeParams = [treeParams]
@@ -733,12 +817,12 @@ def hierarchicalUpdate(curDict: dict, other: dict, replaceLists=False):
       curDict[k] = v
   return curDict
 
-def timedExec(updateFunc: t.Union[t.Generator, t.Callable], interval: int,
+def timedExec(updateFunc: t.Union[t.Generator, t.Callable], interval_ms: int,
               argLst: t.Sequence=None, stopCond: t.Callable=None):
   """
   Iterates over an argument list (or generator) and executes the next action every
-  `interval` ms. Useful for updating GUI elements periodically, e.g. showing a list
-  of images one at a time
+  `interval_ms` ms. Useful for updating GUI elements periodically, e.g. showing a list
+  of images one at a time. Set `interval_ms` to <= 0 to avoid starting right away.
   """
   timer = QtCore.QTimer()
   if argLst is None:
@@ -757,7 +841,8 @@ def timedExec(updateFunc: t.Union[t.Generator, t.Callable], interval: int,
     except StopIteration:
       timer.stop()
   timer.timeout.connect(update)
-  timer.start(interval)
+  if interval_ms > 0:
+    timer.start(interval_ms)
   return timer
 
 def gracefulNext(generator: t.Generator):
