@@ -1,5 +1,6 @@
 import inspect
 import typing as t
+import warnings
 import weakref
 from contextlib import contextmanager
 from enum import Flag, auto
@@ -257,6 +258,8 @@ class ParamEditor(QtWidgets.QDockWidget):
       return None
     elif paramState is None:
       paramState = fns.paramValues(self.params, includeDefaults=includeDefaults or allowOverwriteDefault)
+      # Unnest 'parameters' name
+      paramState = paramState.get('Parameters', {})
       # Nothing changed for this state, no need to do any updating
       applyChanges = False
     # Remove non-useful values
@@ -311,9 +314,9 @@ class ParamEditor(QtWidgets.QDockWidget):
         matches = [p for p in candidateParams if validName(p, kk)]
         if len(matches) == 1:
           matches[0].setValue(vv)
-        elif len(matches) == 0:
-          fns.warnLater(f'No matching parameters for key {kk}. Ignoring.', UserWarning)
-        else:
+        # elif len(matches) == 0:
+        #   warnings.warn(f'No matching parameters for key {kk}. Ignoring.', UserWarning)
+        elif len(matches) > 0:
           raise ValueError(f'Multiple matching parameters for key {kk}:\n'
                            f'{matches}')
 
@@ -372,7 +375,14 @@ class ParamEditor(QtWidgets.QDockWidget):
     return stateName
 
   def _parseStateDict(self, stateName: t.Union[str, Path], stateDict: dict=None):
-    return fns.resolveYamlDict(self.formatFileName(stateName), stateDict)[1]
+    _, stateDict = fns.resolveYamlDict(self.formatFileName(stateName), stateDict)
+    if 'Parameters' in stateDict:
+      warnings.warn('"Parameters" is deprecated for a loaded state. In the future, set options at the'
+                    ' top dictionary level', DeprecationWarning, stacklevel=2)
+      subDict = stateDict.pop('Parameters')
+      if subDict:
+        stateDict.update(subDict)
+    return stateDict
 
   def deleteParamState(self, stateName: str):
     filename = self.formatFileName(stateName)
@@ -723,7 +733,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       dockIdx = tabs.indexOf(self.dockContentsWidget)
       tabs.setCurrentIndex(dockIdx)
     # Necessary on MacOS
-    self.dock.setWindowState(QtCore.Qt.WindowActive)
+    self.dock.setWindowState(QtCore.Qt.WindowState.WindowActive)
     self.dock.raise_()
     self.dock.show()
     # Necessary on Windows
@@ -751,9 +761,11 @@ class ParamEditorDockGrouping(QtWidgets.QDockWidget):
     self.tabs = QtWidgets.QTabWidget(self)
     self.hide()
 
+    if dockName is None:
+      dockName = ''
+
     if editors is None:
       editors = []
-
     if len(dockName) == 0 and len(editors) > 0:
       dockName = editors[0].name
     dockName = dockName.replace('&', '')
@@ -826,7 +838,7 @@ class ParamEditorDockGrouping(QtWidgets.QDockWidget):
     self.raise_()
     self.activateWindow()
     if self.width() < self.biggestMinWidth + 100 and self.win:
-      self.win.resizeDocks([self], [self.biggestMinWidth + 100], QtCore.Qt.Horizontal)
+      self.win.resizeDocks([self], [self.biggestMinWidth + 100], QtCore.Qt.Orientation.Horizontal)
 
 class EditorPropsMixin:
   __groupingName__: str = None
