@@ -90,7 +90,7 @@ class PrjParam:
     :param other: Other :class:`PrjParam` member for comparison
     :return: Whether `self` is less than `other`
     """
-    return str(self) < str(other)
+    return repr(self) < repr(other)
 
   def __eq__(self, other):
     # TODO: Highly naive implementation. Be sure to make this more robust if it needs to be
@@ -100,7 +100,7 @@ class PrjParam:
   def __hash__(self):
     # Since every param within a group will have a unique name, just the name is
     # sufficient to form a proper hash
-    return hash(self.name,)
+    return hash(repr(self),)
 
   def toNumeric(self, data: Sequence, rescale=False, returnMapping=False):
     """
