@@ -1,10 +1,14 @@
 from __future__ import annotations
 
 import contextlib
-import typing
+import html
+import logging
 import typing as t
+import weakref
 from textwrap import wrap
+from traceback import format_exception
 from warnings import warn
+import sys
 
 import cv2 as cv
 import numpy as np
@@ -12,16 +16,16 @@ import pandas as pd
 import pyqtgraph as pg
 from matplotlib import cm
 from matplotlib.pyplot import colormaps
+from numpy.distutils.misc_util import is_sequence
 from pyqtgraph import GraphicsScene
 from pyqtgraph.Qt import QtWidgets, QtGui, QtCore
 from pyqtgraph.graphicsItems.LegendItem import ItemSample
 from pyqtgraph.graphicsItems.ScatterPlotItem import drawSymbol
 from pyqtgraph.graphicsItems.ViewBox.ViewBoxMenu import ViewBoxMenu
 from pyqtgraph.parametertree import Parameter
-from numpy.distutils.misc_util import is_sequence
 
 from utilitys.fns import nameFormatter, pascalCaseToTitle, params_flattened, \
-  dynamicDocstring, getParamChild, hookupParamWidget
+  dynamicDocstring, getParamChild, hookupParamWidget, timedExec
 from utilitys.misc import CompositionMixin
 from utilitys.typeoverloads import FilePath
 from . import params
@@ -78,22 +82,42 @@ else:
       """ Execute a command in the frame of the console widget """
       self._execute(command,False)
 
+# Sometimes, exceptions in the Ipython stack make it difficult to spawn an ipython console even if it is importable
+# Wrap around this issue with a Pyqtgraph console if needed
+def safeSpawnDevConsole(win: QtWidgets.QMainWindow, **locals):
+  """
+  Opens a console that allows dynamic interaction with current variables. If IPython
+  is on your system, a qt console will be loaded. Otherwise, a (less capable) standard
+  pyqtgraph console will be used.
+  """
+  # "dict" default is to use repr instead of string for internal elements, so expanding
+  # into string here ensures repr is not used
+  locals.update(win=win)
+  nsPrintout = [f"{k}: {v}" for k, v in locals.items()]
+  text = f'Starting console with variables:\n' \
+         f'{nsPrintout}'
+  # Broad exception is fine, fallback is good enough. Too many edge cases to properly diagnose when Pycharm's event
+  # loop is sync-able with the Jupyter dev console
+  # noinspection PyBroadException
+  try:
+    console = ConsoleWidget(parent=win, namespace=locals, text=text)
+  except Exception:
+    # Ipy kernel can have issues for many different reasons. Always be ready to fall back to traditional console
+    console = pg.console.ConsoleWidget(parent=win, namespace=locals, text=text)
+  console.setWindowFlags(QtCore.Qt.WindowFlag.Window)
+  console.show()
 
-class ScrollableErrorDialog(QtWidgets.QDialog):
-  def __init__(self, parent: QtWidgets.QWidget=None, notCritical=False,
-               msgWithTrace='', msgWithoutTrace=''):
+class ScrollableMessageDialog(QtWidgets.QDialog):
+  def __init__(self, parent: QtWidgets.QWidget=None, messageType='Information',
+               msg='', detailedMsg=''):
     super().__init__(parent)
     style = self.style()
     self.setModal(True)
 
-    if notCritical:
-      icon = style.standardIcon(style.SP_MessageBoxInformation)
-      self.setWindowTitle('Information')
-    else:
-      icon = style.standardIcon(style.SP_MessageBoxCritical)
-      self.setWindowTitle('Error')
+    styleIcon = getattr(style, f'SP_MessageBox{messageType}')
+    self.setWindowTitle(messageType)
+    self.setWindowIcon(style.standardIcon(styleIcon))
 
-    self.setWindowIcon(icon)
     verticalLayout = QtWidgets.QVBoxLayout(self)
 
 
@@ -105,17 +129,17 @@ class ScrollableErrorDialog(QtWidgets.QDialog):
     scrollLayout = QtWidgets.QVBoxLayout(scrollAreaWidgetContents)
 
     # Set to message with trace first so sizing is correct
-    msgLbl = QtWidgets.QLabel(msgWithTrace, scrollAreaWidgetContents)
-    msgLbl.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse
-                                      | QtCore.Qt.TextSelectableByKeyboard)
-    msgLbl.setTextFormat(QtCore.Qt.PlainText)
-    scrollLayout.addWidget(msgLbl, 0, QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
+    msgLbl = QtWidgets.QLabel(detailedMsg, scrollAreaWidgetContents)
+    msgLbl.setTextInteractionFlags(QtCore.Qt.TextInteractionFlag.TextSelectableByMouse
+                                      | QtCore.Qt.TextInteractionFlag.TextSelectableByKeyboard)
+    msgLbl.setTextFormat(QtCore.Qt.TextFormat.PlainText)
+    scrollLayout.addWidget(msgLbl, 0, QtCore.Qt.AlignmentFlag.AlignLeft|QtCore.Qt.AlignmentFlag.AlignTop)
     scrollArea.setWidget(scrollAreaWidgetContents)
     verticalLayout.addWidget(scrollArea)
 
     btnLayout = QtWidgets.QHBoxLayout()
     ok = QtWidgets.QPushButton('Ok', self)
-    toggleTrace = QtWidgets.QPushButton('Toggle Stack Trace', self)
+    toggleTrace = QtWidgets.QPushButton('Toggle Details', self)
     btnLayout.addWidget(ok)
     btnLayout.addWidget(toggleTrace)
     spacerItem = QtWidgets.QSpacerItem(ok.width(), ok.height(),
@@ -128,22 +152,22 @@ class ScrollableErrorDialog(QtWidgets.QDialog):
     newHeight = max(sh.height(), self.height())
     self.resize(newWidth, newHeight)
 
-    showTrace = False
+    showDetailedMsg = False
     def updateTxt():
-      nonlocal showTrace
-      if showTrace:
-        newText = msgWithTrace.replace('\n', '<br>')
-        msgLbl.setTextFormat(QtCore.Qt.RichText)
+      nonlocal showDetailedMsg
+      if showDetailedMsg:
+        newText = detailedMsg.replace('\n', '<br>')
+        msgLbl.setTextFormat(QtCore.Qt.TextFormat.RichText)
       else:
-        newLines = msgWithoutTrace.splitlines()
+        newLines = msg.splitlines()
         allLines = []
         for line in newLines:
           if line == '': line = [line]
           else: line = wrap(line)
           allLines.extend(line)
         newText = '<br>'.join(allLines)
-        msgLbl.setTextFormat(QtCore.Qt.RichText)
-      showTrace = not showTrace
+        msgLbl.setTextFormat(QtCore.Qt.TextFormat.RichText)
+      showDetailedMsg = not showDetailedMsg
       msgLbl.setText(newText)
 
     self.msgLbl = msgLbl
@@ -205,9 +229,9 @@ class PopupLineEditor(QtWidgets.QLineEdit):
 
   def setModel(self, model: QtCore.QAbstractListModel):
     completer = QtWidgets.QCompleter(model, self)
-    completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
-    completer.setCompletionRole(QtCore.Qt.DisplayRole)
-    completer.setFilterMode(QtCore.Qt.MatchContains)
+    completer.setCaseSensitivity(QtCore.Qt.CaseSensitivity.CaseInsensitive)
+    completer.setCompletionRole(QtCore.Qt.ItemDataRole.DisplayRole)
+    completer.setFilterMode(QtCore.Qt.MatchFlag.MatchContains)
     if self.clearOnComplete:
       completer.activated.connect(lambda: QtCore.QTimer.singleShot(0, self.clear))
 
@@ -244,9 +268,9 @@ class PopupLineEditor(QtWidgets.QLineEdit):
 
     ev: QtGui.QKeyEvent
     key = ev.key()
-    if key == QtCore.Qt.Key_Tab:
+    if key == QtCore.Qt.Key.Key_Tab:
       incAmt = 1
-    elif key == QtCore.Qt.Key_Backtab:
+    elif key == QtCore.Qt.Key.Key_Backtab:
       incAmt = -1
     else:
       return super().event(ev)
@@ -255,14 +279,14 @@ class PopupLineEditor(QtWidgets.QLineEdit):
 
   def focusOutEvent(self, ev: QtGui.QFocusEvent):
     reason = ev.reason()
-    if reason in [QtCore.Qt.TabFocusReason, QtCore.Qt.BacktabFocusReason,
-                  QtCore.Qt.OtherFocusReason]:
+    if reason in [QtCore.Qt.FocusReason.TabFocusReason, QtCore.Qt.FocusReason.BacktabFocusReason,
+                  QtCore.Qt.FocusReason.OtherFocusReason]:
       # Simulate tabbing through completer options instead of losing focus
       self.setFocus()
       completer = self.completer()
       if completer is None:
         return
-      incAmt = 1 if reason == QtCore.Qt.TabFocusReason else -1
+      incAmt = 1 if reason == QtCore.Qt.FocusReason.TabFocusReason else -1
 
       self._chooseNextCompletion(incAmt)
       ev.accept()
@@ -455,10 +479,10 @@ class EasyWidget:
 
   def _resetOpts(self, useSplitter, layout):
     if layout == 'V':
-      orient = QtCore.Qt.Vertical
+      orient = QtCore.Qt.Orientation.Vertical
       layout = QtWidgets.QVBoxLayout
     elif layout == 'H':
-      orient = QtCore.Qt.Horizontal
+      orient = QtCore.Qt.Orientation.Horizontal
       layout = QtWidgets.QHBoxLayout
     else:
       orient = layout = None
@@ -487,9 +511,9 @@ class EasyWidget:
       return
     if self.layout_ is None:
       raise ValueError('Top-level orientation must be set to "V" or "H" before adding children')
-    if self.orient_ == QtCore.Qt.Horizontal:
+    if self.orient_ == QtCore.Qt.Orientation.Horizontal:
       chSuggested = 'V'
-    elif self.orient_ == QtCore.Qt.Vertical:
+    elif self.orient_ == QtCore.Qt.Orientation.Vertical:
       chSuggested = 'H'
     else:
       chSuggested = None
@@ -723,7 +747,6 @@ class MaskCompositor(ImageViewer):
     # Create properties
     # -----
     self.legend = CompositorLegend(offset=(5,5), horSpacing=5, brush='ccce', pen='k')
-    self.legend.setFlag(self.legend.ItemIgnoresTransformations, False)
 
     self.recordDf = pd.DataFrame(columns=['name', 'item', 'scatter', 'opacity']).set_index('name')
 
@@ -950,12 +973,9 @@ class MaskCompositor(ImageViewer):
         item.paint(painter)
     painter.setOpacity(1.0)
 
-    tRect: QtCore.QRectF = self.legend.mapRectToItem(self.imgItem, self.legend.rect())
     if cropToViewbox:
       maxBnd = np.array(self.image.shape[:2][::-1]).reshape(-1,1) - 1
       vRange = np.array(self.viewbox.viewRange()).astype(int)
-      tRect.setX(tRect.x() - vRange[0,0])
-      tRect.setY(tRect.y() - vRange[1,0])
       vRange = np.clip(vRange, 0, maxBnd)
       # Convert range to topleft->bottomright
       pts = QtCore.QPoint(*vRange[:,0]), QtCore.QPoint(*vRange[:,1])
@@ -977,12 +997,24 @@ class MaskCompositor(ImageViewer):
     return saveImg
 
 
-  def _paintLegend(self, painter: QtGui.QPainter, targetRect: QtCore.QRectF):
+  def _paintLegend(self, painter: QtGui.QPainter, floatLegend=False):
     oldPos = self.legend.pos()
     try:
       exportScene = GraphicsScene(parent=None)
+      oldScale = max(np.clip(0.5/np.array(self.imgItem.pixelSize()), 1, np.inf))
       exportScene.addItem(self.legend)
-      exportScene.render(painter, targetRect)
+      self.legend.setScale(oldScale)
+      # Legend does not scale itself, handle this
+      painter.save()
+      if floatLegend:
+        viewPos = self.viewbox.viewRect()
+        viewX = max(viewPos.x(), 0)
+        viewY = max(viewPos.y(), 0)
+        imPos = self.legend.mapRectToItem(self.imgItem, self.legend.rect())
+        painter.translate(int(imPos.x() - viewX), int(imPos.y() - viewY))
+      exportScene.render(painter)
+      self.legend.setScale(1/oldScale)
+      painter.restore()
     finally:
       self.legend.setParentItem(self.plotItem)
       self.legend.autoAnchor(oldPos, relative=False)
@@ -1010,12 +1042,12 @@ class PandasTableModel(QtCore.QAbstractTableModel):
   def columnCount(self, parent=None):
     return self.df.shape[1]
 
-  def data(self, index, role=QtCore.Qt.DisplayRole):
+  def data(self, index, role=QtCore.Qt.ItemDataRole.DisplayRole):
     if index.isValid():
       value = self.df.iloc[index.row(), index.column()]
-      if role == QtCore.Qt.DisplayRole:
+      if role == QtCore.Qt.ItemDataRole.DisplayRole:
         return str(value)
-      elif role == QtCore.Qt.EditRole:
+      elif role == QtCore.Qt.ItemDataRole.EditRole:
         return value
     return None
 
@@ -1036,12 +1068,12 @@ class PandasTableModel(QtCore.QAbstractTableModel):
     self.sigDataChanged.emit()
     return True
 
-  def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
-    if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
+  def headerData(self, section, orientation, role=QtCore.Qt.ItemDataRole.DisplayRole):
+    if orientation == QtCore.Qt.Orientation.Horizontal and role == QtCore.Qt.ItemDataRole.DisplayRole:
       return self.df.columns[section]
 
   def flags(self, index: QtCore.QModelIndex) -> QtCore.Qt.ItemFlags:
-    return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
+    return QtCore.Qt.ItemFlag.ItemIsEnabled | QtCore.Qt.ItemFlag.ItemIsSelectable
 
   def addDfRows(self, rowData: pd.DataFrame, addType = PrjEnums.ADD_TYPE_NEW, emitChange=True):
     toEmit = self.defaultEmitDict.copy()
@@ -1173,4 +1205,166 @@ def makeDummySignal(obj, sigName: str, capture=False):
     setattr(obj, sigName, newSig)
     yield newSig
   finally:
-    setattr(obj, sigName, oldSig)
\ No newline at end of file
+    setattr(obj, sigName, oldSig)
+
+
+class QtAppHandler(logging.Handler):
+  _weakWin: weakref.ReferenceType = None
+
+  def __init__(self,
+              level,
+              win: QtWidgets.QMainWindow=None,
+              exceptionsOnly=False) -> None:
+    super().__init__(level)
+    self.attachWinRef(win)
+    self.exceptionsOnly = exceptionsOnly
+
+  def attachWinRef(self, win=None):
+    if win is not None:
+      win = weakref.ref(win)
+    self._weakWin = win
+
+  def getWin(self) -> t.Optional[QtWidgets.QMainWindow]:
+    if self._weakWin is None:
+      return None
+    return self._weakWin()
+
+class DialogHandler(QtAppHandler):
+  def emit(self, record: logging.LogRecord):
+    if record.exc_info:
+      msg = html.escape(str(record.exc_info[1]))
+      excStr = ''.join(format_exception(*record.exc_info))
+      detailed = html.escape(excStr)
+    else:
+      msg = super().format(record)
+      detailed = getattr(record, 'detailed', msg)
+    dlgType = 'Critical' if record.levelno == logging.CRITICAL else 'Information'
+
+    dlg = ScrollableMessageDialog(self.getWin(), dlgType, msg, detailed)
+    def doExec():
+      dlg.exec()
+    # Using function instead of directly connecting to dlg.exec keeps garbage from being collected just long enough
+    QtCore.QTimer.singleShot(0, doExec)
+
+  def filter(self, record: logging.LogRecord):
+    """Optionally only activates this handler if the message came from an exception"""
+    if not self.exceptionsOnly:
+      return super().filter(record)
+    try:
+      return len(record.exc_info) > 1
+    except Exception:
+      return False
+
+class TimedMessageHandler(QtAppHandler):
+
+  def __init__(self, *args, defaultMsgtimeout_ms=3000, maxLevel=None, **kwargs) -> None:
+    super().__init__(*args, **kwargs)
+    self.defaultMsgTimeout = defaultMsgtimeout_ms
+    if maxLevel is None:
+      maxLevel = sys.maxsize
+    self.maxLevel = maxLevel
+    
+
+  def emit(self, record: logging.LogRecord):
+    win = self.getWin()
+    if win is None:
+      return
+    if hasattr(record, 'msgTimeout'):
+      msgTimeout = record.msgTimeout
+    else:
+      msgTimeout = self.defaultMsgTimeout
+    self.makeNotification(self.format(record), msgTimeout)
+
+  def filter(self, record: logging.LogRecord):
+    return record.levelno <= self.maxLevel and logging.Handler.filter(self, record)
+
+  def makeNotification(self, msg: str, timeout_ms: int=None):
+    raise NotImplementedError
+
+class StatusBarHandler(TimedMessageHandler):
+  def makeNotification(self, msg: str, timeout_ms: int=None):
+    self.getWin().statusBar().showMessage(msg, timeout_ms)
+
+class FadeNotifyHandler(TimedMessageHandler):
+  updateFps = 10
+
+  def __init__(self, *args, **kwargs):
+    super().__init__(*args, **kwargs)
+    # If references aren't kept to labels, they will caused wrapped c++ deletion errors
+    # List instead of single ref since multiple messages can exist at once
+    # Dict to also associate animations with their window
+    self.msgRefs = {}
+
+  def makeNotification(self, msg: str, timeout_ms: int=None):
+    """
+    Creates a fading green dialog that shows the message and disappears after timeout
+    """
+    style = """\
+    QLabel {
+      background-color: #0f0;
+      border: 1px solid #000;
+      color: #000;
+    };
+    """
+    fadeMsg = QtWidgets.QLabel(msg, self.getWin())
+    fadeMsg.setStyleSheet(style)
+    # For some reason, width doesn't set properly here
+    fadeMsg.setFixedWidth(fadeMsg.sizeHint().width())
+    fadeMsg.setAlignment(QtCore.Qt.AlignCenter)
+    fadeMsg.setAttribute(QtCore.Qt.WidgetAttribute.WA_TransparentForMouseEvents)
+    parent: QtWidgets.QMainWindow = self.getWin()
+    if parent is not None:
+      parentPos = parent.pos()
+      metrics = fadeMsg.fontMetrics()
+      padding = (parent.width() - metrics.width(msg))//2
+      msgX = padding + parentPos.x()
+      # Make sure multiple messages don't overlap
+      msgY = int(0.05*parent.height()) + parentPos.y()
+      msgY += self.getUniqueLblOffset(msgY)
+      fadeMsg.move(msgX, msgY)
+    anim = self.makeAnimation(fadeMsg, timeout_ms)
+    # 10 fps
+    anim.start(1000//self.updateFps)
+    fadeMsg.show()
+
+  def getUniqueLblOffset(self, constHeight):
+    allYPos = np.array([l.pos().y() - constHeight for l in self.msgRefs])
+    if not len(allYPos):
+      return 0
+    height = next(iter(self.msgRefs)).height()
+    offset = 0
+    recalc = True
+    while recalc:
+      offset += 0.8*height
+      recalc = np.min(np.abs(allYPos - offset)) < 0.01
+    return offset
+
+
+  def closeMsgWidget(self, msgWidget):
+    """
+    Closes the specified messages and deletes corresponding references
+    """
+    msgWidget.hide()
+    msgWidget.deleteLater()
+    self.msgRefs[msgWidget].deleteLater()
+    del self.msgRefs[msgWidget]
+
+  def makeAnimation(self, widget, timeout_ms):
+    """
+    The default method of using a PropertyAnimation doesn't repaint the widget, so do it a quick and dirty way
+    using timers manually
+    """
+    def changeOpacity():
+      for opacity in np.linspace(1, 0, int(timeout_ms/1000*self.updateFps)):
+        effect.setOpacity(opacity)
+        # widget.update()
+        yield
+      # Animation done, wait one tick to prevent overlaps
+      yield
+      self.closeMsgWidget(widget)
+    # Don't start right away
+    effect = QtWidgets.QGraphicsOpacityEffect(widget)
+    widget.setGraphicsEffect(effect)
+    anim = timedExec(changeOpacity, interval_ms=0)
+    self.msgRefs[widget] = anim
+    return anim
\ No newline at end of file
